package com.nisum.ascend.promotions.controller;

import com.nisum.ascend.promotions.dto.PromotionDto;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.reactive.AutoConfigureWebTestClient;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.junit.jupiter.SpringExtension;
import org.springframework.test.web.reactive.server.WebTestClient;
import reactor.core.publisher.Mono;

import java.util.List;

import static org.junit.jupiter.api.Assertions.*;

@ExtendWith(SpringExtension.class)
@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
@DirtiesContext
@AutoConfigureWebTestClient
@ActiveProfiles("test")
@Slf4j
class PromotionsControllerTest {
    @Autowired
    private WebTestClient webTestClient;

    @BeforeEach
    void setUp() {
    }

    WebTestClient.ResponseSpec getPromotionWebClient(String promoId) {
        return webTestClient.get().uri("/api/promos/" + promoId).exchange();
    }

    @Test
    void getAllPromotions() {
        webTestClient.get().uri("/api/promos").exchange()
                .expectStatus().isOk()
                .expectHeader().contentType(MediaType.APPLICATION_JSON_VALUE)
                .expectBodyList(PromotionDto.class)
                .consumeWith(promo ->{
                    List<PromotionDto> promos = promo.getResponseBody();
                    assert promos != null;
                    promos.forEach(p ->{
                        assertNotNull(p.getPromotionId());
                        assertNotNull(p.getProductSku());
                        System.out.println(p.getPromotionId());
                    });
                });
    }

    @Test
    void getPromotionById() {
        getPromotionWebClient("0003")
                .expectStatus().isOk()
                .expectHeader().contentType(MediaType.APPLICATION_JSON_VALUE)
                .expectBody()
                .jsonPath("promotionId", "0003");

    void createPromotion(){
        Promotion promotion = new Promotion("50OFF","SH1234", (float) 0.5,5);
        webTestClient.post().uri("/api/promos")
                .contentType(MediaType.valueOf(MediaType.APPLICATION_JSON_VALUE))
                .body(Mono.just(promotion),Promotion.class)
                .exchange()
                .expectStatus().isOk()
                .expectBody()
                .jsonPath("$.promotionId","50OFF");
    }

    @Test
    void getPromotionsByProductSku(){
        String skuWithOnePromo = "AFP-1";
        String skuWithMultiplePromos = "AFP-2";

        webTestClient.get()
                .uri(uriBuilder -> uriBuilder
                        .path("/api/promos")
                        .queryParam("sku", skuWithOnePromo)
                        .build())
                .exchange()
                .expectStatus().isOk()
                .expectHeader().contentType(MediaType.APPLICATION_JSON_VALUE)
                .expectBodyList(PromotionDto.class)
                .hasSize(1)
                .consumeWith(promo ->{
                    List<PromotionDto> promos = promo.getResponseBody();
                    assert promos != null;
                    promos.forEach(p -> assertEquals(skuWithOnePromo, p.getProductSku()));
                });

        webTestClient.get()
                .uri(uriBuilder -> uriBuilder
                        .path("/api/promos")
                        .queryParam("sku", skuWithMultiplePromos)
                        .build())
                .exchange()
                .expectStatus().isOk()
                .expectHeader().contentType(MediaType.APPLICATION_JSON_VALUE)
                .expectBodyList(PromotionDto.class)
                .hasSize(2)
                .consumeWith(promo ->{
                    List<PromotionDto> promos = promo.getResponseBody();
                    assert promos != null;
                    promos.forEach(p -> assertEquals(skuWithMultiplePromos, p.getProductSku()));
                });
    }
}